/***************************************************************************
 *   Copyright (C) 2007-2010 Mike Kinghan, imk@strudl.org            *
 *   All rights reserved.                                                  *
 *                                                                         *
 *   Contributed originally by Mike Kinghan, imk@strudl.org                *
 *                                                                         *
 *   Redistribution and use in source and binary forms, with or without    *
 *   modification, are permitted provided that the following conditions    *
 *   are met:                                                              *
 *                                                                         *
 *   Redistributions of source code must retain the above copyright        *
 *   notice, this list of conditions and the following disclaimer.         *
 *                                                                         *
 *   Redistributions in binary form must reproduce the above copyright     *
 *   notice, this list of conditions and the following disclaimer in the   *
 *   documentation and/or other materials provided with the distribution.  *
 *                                                                         *
 *   Neither the name of Symbian Software Ltd. nor the names of its        *
 *   contributors may be used to endorse or promote products derived from  *
 *   this software without specific prior written permission.              *
 *                                                                         *
 *   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS   *
 *   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT     *
 *   LIMITED TO, THE IMPLIED WARRANTIES OF  MERCHANTABILITY AND FITNESS    *
 *   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE        *
 *   COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,   *
 *   INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,  *
 *   BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS *
 *   OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED    *
 *   AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,*
 *   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF *
 *   THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH  *
 *   DAMAGE.                                                               *
 *                                                                         *
 **************************************************************************/
#include "configured_symbols.h"
#include "report.h"
#include "chew.h"
#include "args.h"
#include "ptr_set.h"
#include "evaluator.h"

/*!\ingroup configured_symbols_component configured_symbols_interface configured_symbols_internals
	\file configured_symbols.c
	This file implements the Configured Symbols component
*/

/*!\ingroup configured_symbols_internals_state_utils*/
/*@{*/
NO_PRIVATE_STATE(configured_symbols);
/*@}*/

/*!\addtogroup configured_symbols_internals_state_utils*/
/*@{*/
IMPLEMENT(configured_symbols,USER_INITABLE);

DEFINE_USER_INIT(configured_symbols)(STATE_T(configured_symbols) * st_state)
{
    st_state->sym_tab = ptr_set_new((comparator_t)symbol_compare,
                                    (dtor_t)symbol_dispose,
                                    (cloner_t)symbol_copy);
}

DEFINE_USER_FINIS(configured_symbols)(STATE_T(configured_symbols) * st_state)
{
    ptr_set_dispose(st_state->sym_tab);
}
/*@}*/

/* API*/

symbol_const_h
configured_symbol_seek_const(char const **cpp)
{
    assert(cpp);
    canonical_string_h symname = canonical_identifier(cpp);
    if (!*canonical_string_text(symname)) {
        bail(	GRIPE_NOT_IDENTIFIER,
                "Identifier needed instead of \"%s\"",
                *cpp);
    }
    return configured_symbol_match_const(symname);
}

symbol_h
configured_symbol_seek(char const **cpp)
{
    return (symbol_h)configured_symbol_seek_const(cpp);
}

symbol_const_h
configured_symbol_match_const(canonical_string_const_h symname)
{
    symbol_const_h sym = NULL;
    assert(symname);
    char const *name = canonical_string_text(symname);
    if (name[0]) {
        sym = ptr_set_search(GET_PUBLIC(configured_symbols,sym_tab),
                             name,strlen(name));
        if (sym) {
            debug(DBG_18, symbol_name(sym),symbol_definition(sym));
        }
    }
    return sym;
}

symbol_h
configured_symbol_match(canonical_string_const_h symname)
{
    return (symbol_h)configured_symbol_match_const(symname);
}


void
configured_symbol_add(bool define, char const *symstr)
{
    char const *val;
    char const *name_end;
    locator_t loc;
    symbol_h sym;
    assert(symstr);
    name_end = chew_symbol(symstr);
    val = chew_macro_call(name_end);
    sym = symbol_new(symstr,val - symstr);
    loc  = ptr_set_insert(GET_PUBLIC(configured_symbols,sym_tab),sym);
    if (SOMEWHERE(loc)) {
        symbol_h sym = (symbol_h)POINTER(loc);
        if (define) { /* -D*/
            if (val > name_end) {
                bail(GRIPE_CANT_CONFIGURE_MACROS,
                     "Sorry. Can't -D macro expansions: \"%s\"",symstr - 2);
            }
            if (*val == '=') {
                ++val;
                symbol_define(sym,&val);
            } else if (*val == '\0') {
                symbol_define(sym,NULL);
            } else {	/* Invalid*/
                bail(GRIPE_GARBAGE_ARG,"Garbage in argument \"%s\"",symstr - 2);
            }
        } else {	/* -U*/
            if (*val != '\0') { /* Invalid*/
                bail(GRIPE_GARBAGE_ARG,"Garbage in argument \"%s\"",symstr - 2);
            }
        }
    } else {
        /* Duplicate arg*/
        diagnose_args_error(symstr,val,(symbol_const_h)~loc);
    }
}

void
configured_symbol_tally(symbol_h symbol)
{
    if (!symbol_seen(symbol)) {
        symbol_resolve(symbol);
        report_symbol(symbol);
    } else if (!GET_PUBLIC(args,list_only_once)) {
        report_symbol(symbol);
    }
}



/* EOF*/
