#ifndef CONFIGURED_SYMBOLS_H
#define CONFIGURED_SYMBOLS_H

/***************************************************************************
 *   Copyright (C) 2007-2010 Mike Kinghan, imk@strudl.org            *
 *   All rights reserved.                                                  *
 *                                                                         *
 *   Contributed originally by Mike Kinghan, imk@strudl.org                *
 *                                                                         *
 *   Redistribution and use in source and binary forms, with or without    *
 *   modification, are permitted provided that the following conditions    *
 *   are met:                                                              *
 *                                                                         *
 *   Redistributions of source code must retain the above copyright        *
 *   notice, this list of conditions and the following disclaimer.         *
 *                                                                         *
 *   Redistributions in binary form must reproduce the above copyright     *
 *   notice, this list of conditions and the following disclaimer in the   *
 *   documentation and/or other materials provided with the distribution.  *
 *                                                                         *
 *   Neither the name of Symbian Software Ltd. nor the names of its        *
 *   contributors may be used to endorse or promote products derived from  *
 *   this software without specific prior written permission.              *
 *                                                                         *
 *   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS   *
 *   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT     *
 *   LIMITED TO, THE IMPLIED WARRANTIES OF  MERCHANTABILITY AND FITNESS    *
 *   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE        *
 *   COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,   *
 *   INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,  *
 *   BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS *
 *   OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED    *
 *   AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,*
 *   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF *
 *   THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH  *
 *   DAMAGE.                                                               *
 *                                                                         *
 **************************************************************************/

#include "state_utils.h"
#include "ptr_set.h"
#include "symbol.h"

/*!\ingroup configured_symbols_component configured_symbols_interface
	\file configured_symbols.h
	This file provides the Configured Symbols component interface
*/

/*!\addtogroup configured_symbols_interface*/
/*@{*/

/*! Parse an identifier and look it up in configured symbol table.
	\param cpp	On entry, pointer to the address of the text to be parsed as an identifier.
                On entry, stores the adress of the first scanned character that is not
                part of an identifier.
	\return A reference to the matched symbol in the configured symbol table,
			if found, else NULL
*/
extern symbol_h
configured_symbol_seek(char const **cpp);

/*! Parse an identifier and look it up in configured symbol table.
	\param cpp	On entry, pointer to the address of the text to be parsed as an identifier.
                On entry, stores the adress of the first scanned character that is not
                part of an identifier.
	\return A const reference to the matched symbol in the configured symbol
			table,if found, else NULL
*/
extern symbol_const_h
configured_symbol_seek_const(char const **cpp);

/*! Lookup an identifier in the configured symbol table.
	\param symname	The identifier to match.
	\return A reference to the matched symbol in the configured symbol table,
			if found, else NULL
*/
extern symbol_h
configured_symbol_match(canonical_string_const_h symname);

/*! Lookup an identifier in the configured symbol table.
	\param symname	The identifier to match with symbols in the table.
	\return A const reference to the matched symbol in the configured
			symbol table,if found, else NULL
*/
extern symbol_const_h
configured_symbol_match_const(canonical_string_const_h symname);

/*! Add a configured symbol to the configured symbol table.
	\param	define	Is this symbol configured as defined, or undefined?
	\param	sym	 Null-terminated symbol configuration string, of the form:

	\e SYMBOL[=VAL]
*/
extern void
configured_symbol_add(bool define, char const *sym);

/*!	Tally an occurrence of an unconfigured symbol.
	\param	symbol	The symbol to be tallied.

	If \e symbol has not yet been seen then it is flagged as seen.

	The occurrence of the symbol is reported on stdout if the
	symbol has not already been seen or if \c --once is not in
	effect.
*/
extern void
configured_symbol_tally(symbol_h symbol);

/*@}*/

/*!\ingroup configured_symbols_interface_state_utils*/
/*@{*/
/*! The public state of the Configured Symbols component*/
PUBLIC_STATE_DEF(configured_symbols)
{
    ptr_set_h sym_tab;
    /*!< Handle to the Configured Symbols table*/
}
PUBLIC_STATE_T(configured_symbols);
/*@}*/

/*!\addtogroup configured_symbols_interface_state_utils*/
/*@{*/
IMPORT(configured_symbols);
/*@}*/


#endif /* EOF*/
