#ifndef DIRECTIVE_TALLY_H
#define DIRECTIVE_TALLY_H
/***************************************************************************
 *   Copyright (C) 2007-2010 Mike Kinghan, imk@strudl.org            *
 *   All rights reserved.                                                  *
 *                                                                         *
 *   Contributed originally by Mike Kinghan, imk@strudl.org                *
 *                                                                         *
 *   Redistribution and use in source and binary forms, with or without    *
 *   modification, are permitted provided that the following conditions    *
 *   are met:                                                              *
 *                                                                         *
 *   Redistributions of source code must retain the above copyright        *
 *   notice, this list of conditions and the following disclaimer.         *
 *                                                                         *
 *   Redistributions in binary form must reproduce the above copyright     *
 *   notice, this list of conditions and the following disclaimer in the   *
 *   documentation and/or other materials provided with the distribution.  *
 *                                                                         *
 *   Neither the name of Symbian Software Ltd. nor the names of its        *
 *   contributors may be used to endorse or promote products derived from  *
 *   this software without specific prior written permission.              *
 *                                                                         *
 *   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS   *
 *   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT     *
 *   LIMITED TO, THE IMPLIED WARRANTIES OF  MERCHANTABILITY AND FITNESS    *
 *   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE        *
 *   COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,   *
 *   INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,  *
 *   BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS *
 *   OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED    *
 *   AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,*
 *   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF *
 *   THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH  *
 *   DAMAGE.                                                               *
 *                                                                         *
 **************************************************************************/

#include "state_utils.h"
#include "canonical_string.h"

/*!\ingroup directives_tally_component directives_tally_interface
 *\file directives_tally.h
 * This file provides the Directive Tally component interface
*/

/*!\addtogroup directives_tally_interface*/
/*@{*/

/*! Symbolic constants for types of directives */
typedef enum {
    HASH_UNKNOWN,	/*!< An unknown directive */
    HASH_IF,		/*!< A hash-if directive */
    HASH_IFDEF,		/*!< An hash-ifdef directive */
    HASH_IFNDEF,	/*!< An hash-ifndef directive */
    HASH_ELSE,		/*!< An hash-else directive */
    HASH_ELIF,		/*!< An hash-elif directive */
    HASH_ENDIF,		/*!< An hash-endif directive */
    HASH_DEFINE,	/*!< An hash-define directive */
    HASH_UNDEF,		/*!< An hash-undef directive */
    HASH_INCLUDE,	/*!< An hash-include directive */
    HASH_PRAGMA,	/*!< A hash-pragma directive */
    HASH_ERROR,		/*!< An #error directive */
    HASH_COUNT		/*!< Number of previous members of this \c enum*/
} directive_type_t;


/*!	Tally an occurrence of a preprocessor directive argument.
	\param	cl A canonical string containg the directive argument.
	\param	directive_type	The type of directive found with \e arg.
	\return The terminal address of the argument parsed at \e arg.

	Arguments of directives are stored, differentiated by directive type,
 	in the lookup tables in \e canonical_string_h form. \e arg is looked up by
	forming a \e canonical_string_h from \e arg and
	searching the table associated with \e directive_type for that cleaned
	line.

	If no argument matching \e arg exists in in the lookup table
	for \e directive_type then the new \e canonical_string_h is added to the
	table.

	The occurrence of the \c arg is reported on stdout if the
	same argument has not already been recorded for \e directive_type or if
	 \c --once is not in effect.
*/
extern void
directive_tally(canonical_string_const_h cl,
                directive_type_t directive_type);

/*! Determine the type of a putative directive.
	\param	keyword	On entry, a pointer to the address of the putative directve.
            On return, stores the address of the character just passed the matched
            directive, if any is matched; otherwise unchanged.
	\return A directive_type_t indicating the type of the directive,
		or HASH_UNKOWN if no legal directive is unambigously identified.
*/
extern directive_type_t
get_directive_type(char const **keyword);

/*! Get the preprocessor keyword associated with a given directive type.
	\param	directive_type Type of directive for which the keyword is required.
	\return	Pointer to the keyword for this directive type.
*/
extern char const *
get_directive_keyword(directive_type_t directive_type);

/*@}*/

/*! \addtogroup chew_interface_state_utils*/
/*@{*/
IMPORT_INITOR(directives_tally);
/*@}*/

#endif /* EOF*/
