/***************************************************************************
 *   Copyright (C) 2007-2010 Mike Kinghan, imk@strudl.org            *
 *   All rights reserved.                                                  *
 *                                                                         *
 *   Originally contributed by Mike Kinghan, imk@strudl.org.               *
 *                                                                         *
 *   Redistribution and use in source and binary forms, with or without    *
 *   modification, are permitted provided that the following conditions    *
 *   are met:                                                              *
 *                                                                         *
 *   Redistributions of source code must retain the above copyright        *
 *   notice, this list of conditions and the following disclaimer.         *
 *                                                                         *
 *   Redistributions in binary form must reproduce the above copyright     *
 *   notice, this list of conditions and the following disclaimer in the   *
 *   documentation and/or other materials provided with the distribution.  *
 *                                                                         *
 *   Neither the name of Symbian Software Ltd. nor the names of its        *
 *   contributors may be used to endorse or promote products derived from  *
 *   this software without specific prior written permission.              *
 *                                                                         *
 *   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS   *
 *   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT     *
 *   LIMITED TO, THE IMPLIED WARRANTIES OF  MERCHANTABILITY AND FITNESS    *
 *   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE        *
 *   COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,   *
 *   INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,  *
 *   BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS *
 *   OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED    *
 *   AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,*
 *   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF *
 *   THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH  *
 *   DAMAGE.                                                               *
 *                                                                         *
 **************************************************************************/
#include "eval_result.h"
#include <stdlib.h>
#include <limits.h>

/* API */

/*!\ingroup eval_result_component eval_result_interface eval_result_internals
 *\file eval_result.c
 * This file implements the Evaluation Result component
*/

/*! \ingroup eval_result_internals */
/*@{*/

/*@}*/

/* API */


/*! Constant \em int_spec_t representing falsity */
int_spec_t const int_spec_false = {INT_UNION_INITOR,INT_INT};

/*! Constant \em int_spec_t representing truth */
int_spec_t const int_spec_true = {{1},INT_INT};

void
eval_result_set_value(eval_result_t *result, int_spec_t const * int_spec)
{
    result->value = *int_spec;
    SET_FLAGS(*result,int_spec->val.ull ? EVAL_TRUE : EVAL_FALSE);
}

void
eval_result_clear(eval_result_t *result)
{
    static int_spec_t initor = INT_SPEC_INITOR;
    if (result) {
        if (result->sym_name) {
            free(result->sym_name);
            result->sym_name = NULL;
        }
        if (result->sym_def) {
            canonical_string_dispose(result->sym_def);
            result->sym_def = NULL;
        }
        if (result->most_resolved) {
            canonical_string_dispose(result->most_resolved);
            result->most_resolved = NULL;
        }
        result->value = initor;
        result->flags = EVAL_PENDING;
    }
}


bool
eval_result_parsed(eval_result_t *result)
{
    return RESOLVED(*result) || KEEP(*result);
}



/* EOF */

