#ifndef EVALUATOR_H
#define EVALUATOR_H

/***************************************************************************
 *   Copyright (C) 2004, 2006 Symbian Software Ltd.                        *
 *   All rights reserved.
 *   Copyright (C) 2002, 2003 Tony Finch <dot@dotat.at>.                   *
 *   All rights reserved.                                                  *
 *   Copyright (C) 1985, 1993 The Regents of the University of California. *
 *   All rights reserved.                                                  *
 *   Copyright (C) 2007-2010 Mike Kinghan, imk@strudl.org            *
 *   All rights reserved.                                                  *
 *                                                                         *
 *   Contributed by Mike Kinghan, imk@strudl.org, derived from the code    *
 *   of Tony Finch                                                         *
 *                                                                         *
 *   Redistribution and use in source and binary forms, with or without    *
 *   modification, are permitted provided that the following conditions    *
 *   are met:                                                              *
 *                                                                         *
 *   Redistributions of source code must retain the above copyright        *
 *   notice, this list of conditions and the following disclaimer.         *
 *                                                                         *
 *   Redistributions in binary form must reproduce the above copyright     *
 *   notice, this list of conditions and the following disclaimer in the   *
 *   documentation and/or other materials provided with the distribution.  *
 *                                                                         *
 *   Neither the name of Symbian Software Ltd. nor the names of its        *
 *   contributors may be used to endorse or promote products derived from  *
 *   this software without specific prior written permission.              *
 *                                                                         *
 *   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS   *
 *   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT     *
 *   LIMITED TO, THE IMPLIED WARRANTIES OF  MERCHANTABILITY AND FITNESS    *
 *   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE        *
 *   COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,   *
 *   INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,  *
 *   BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS *
 *   OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED    *
 *   AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,*
 *   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF *
 *   THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH  *
 *   DAMAGE.                                                               *
 *                                                                         *
 **************************************************************************/

#include "state_utils.h"
#include "eval_result.h"
#include "symbol.h"

/*!\ingroup evaluator_component evaluator_interface
*  \file evaluator.h
   This file provides the Evaluator component interface
*/

/*! \addtogroup evaluator_interface*/
/*@{*/

/*! Enumeration of types of input lines*/
typedef enum {
    /*! An hash-if that we can't resolve*/
    LT_IF,
    /*! A true hash-if*/
    LT_TRUE,
    /*! A false hash-if*/
    LT_FALSE,
    /*! An hash-elif that we can't resolve*/
    LT_ELIF,
    /*! A true hash-elif*/
    LT_ELTRUE,
    /*! A false hash-elif*/
    LT_ELFALSE,
    /*! An hash-else that we can't resolve */
    LT_ELSE,
    /*! An hash-endif*/
    LT_ENDIF,
    /*! A non-directive line*/
    LT_PLAIN,
    /*! End of file*/
    LT_EOF,
    /*! Not a line type. This is the number of line types for which we
    	handlers exist in the transition table. Subsequent line types
    	are handled outside the transition table.
    */
    LT_TABLE_HANDLERS,
    /*! A hash-define that is independant of the \c --define and \c --undefine
        options and is kept
    */
    LT_CONSISTENT_DEFINE_KEEP,
    /*! A hash-define that echoes a \c --define option and is dropped*/
    LT_CONSISTENT_DEFINE_DROP,
    /*! A hash-define that contradicts an \c --undefine option*/
    LT_CONTRADICTORY_DEFINE,
    /*! A hash-define that differently redefines a \c --define option*/
    LT_DIFFERING_DEFINE,
    /*! An hash-undef that is independant of the \c --define and \c --undefine
        options and is kept
    */
    LT_CONSISTENT_UNDEF_KEEP,
    /*! An hash-undefine that echoes a \c --undefine option and is dropped*/
    LT_CONSISTENT_UNDEF_DROP,
    /*! An hash-undef that contradicts a \c --define option*/
    LT_CONTRADICTORY_UNDEF,
    /*! An include directive that is kept */
    LT_INCLUDE_KEEP,
    /*! An include directive that is dropped. */
    LT_INCLUDE_DROP
} line_type_t;

/*! Parse the current input line and return its line type.*/
extern line_type_t
eval_line(void);

/*! Evaluate text within a symbol definition.
	\param	pdef	On entry points to the address from whih to
		parse definitional text. On exit receives the address of the
		unparsed remainder of the text.
	\return An \e eval_result_t representing the result of evaluating
	the definitional text.

	The function is a wrapper for \e eval_table that controls entry
	and exit from the state of parsing a symbol definition as opposed
	to input source. When parsing a symbol definition diagnostics are
	suppressed that would be emitted if parsing input source.

*/
extern eval_result_t
eval_definition_text(char const **pdef);

/*@}*/

/*! \addtogroup evaluator_interface_state_utils*/
/*@{*/
IMPORT_INITOR(evaluator);
IMPORT_FINITOR(evaluator);
/*@}*/


#endif /* EOF*/
