/***************************************************************************
 *   Copyright (C) 2007-2010 Mike Kinghan, imk@strudl.org            *
 *   All rights reserved.                                                  *
 *                                                                         *
 *   Contributed originally by Mike Kinghan, imk@strudl.org                *
 *                                                                         *
 *   Redistribution and use in source and binary forms, with or without    *
 *   modification, are permitted provided that the following conditions    *
 *   are met:                                                              *
 *                                                                         *
 *   Redistributions of source code must retain the above copyright        *
 *   notice, this list of conditions and the following disclaimer.         *
 *                                                                         *
 *   Redistributions in binary form must reproduce the above copyright     *
 *   notice, this list of conditions and the following disclaimer in the   *
 *   documentation and/or other materials provided with the distribution.  *
 *                                                                         *
 *   Neither the name of Symbian Software Ltd. nor the names of its        *
 *   contributors may be used to endorse or promote products derived from  *
 *   this software without specific prior written permission.              *
 *                                                                         *
 *   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS   *
 *   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT     *
 *   LIMITED TO, THE IMPLIED WARRANTIES OF  MERCHANTABILITY AND FITNESS    *
 *   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE        *
 *   COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,   *
 *   INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,  *
 *   BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS *
 *   OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED    *
 *   AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,*
 *   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF *
 *   THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH  *
 *   DAMAGE.                                                               *
 *                                                                         *
 **************************************************************************/
#include "includes_tally.h"
#include "hash_include.h"
#include "args.h"
#include "report.h"

/*!\ingroup includes_tally_component includes_tally_interface includes_tally_internals
	\file includes_tally.c
	This file implements the Includes Tally component
*/

/*!\ingroup includes_tally_internals_state_utils*/
/*@{*/
NO_PRIVATE_STATE(includes_tally);
/*@}*/

/*!\addtogroup includes_tally_internals_state_utils*/
/*@{*/
IMPLEMENT(includes_tally,USER_INITABLE);

DEFINE_USER_INIT(includes_tally)(STATE_T(includes_tally) * st_state)
{
    st_state->includes_tab =
        ptr_set_new((comparator_t)hash_include_compare,
                    (dtor_t)hash_include_dispose,
                    (cloner_t)hash_include_copy);

}

DEFINE_USER_FINIS(includes_tally)(STATE_T(includes_tally) * st_state)
{
    ptr_set_dispose(st_state->includes_tab);
}
/*@}*/

/* Helpers */

/*! \addtogroup includes_tally_internals*/

/*@{*/

/* Helpers */

/*!	Lookup the includes table for a hash-include argument.
	\param	arg	The start of the hash-include argument.
	\param	arglen	The length of the hash-include argument.
	\return If a hash-include argument with the given name is found then it
is
	returned. Otherwise a new hash-include argument with that name. A new
hash-include argument
	can be distinguished by calling \e hash_include_seen(), which will return
	false if the hash-include argument is new.
*/
static hash_include_h
hash_include_lookup(char const *arg, size_t arglen)
{
    hash_include_h inc_arg =
        ptr_set_search(GET_PUBLIC(includes_tally,includes_tab),
                       arg,arglen);
    return inc_arg;
}

/*!	Add a hash-include argument to the includes table.
	\param	inc_arg	The argument to be added.
*/
static void
hash_include_add(hash_include_h inc_arg)
{
    hash_include_memorise(inc_arg);
    ptr_set_insert(GET_PUBLIC(includes_tally,includes_tab),inc_arg);
}

/*@}*/

/* API *********************************************************************/

void
hash_include_tally(	char const *arg,
                    size_t arglen,
                    canonical_string_const_h sym_def)
{
    hash_include_h inc_arg = hash_include_lookup(arg,arglen);
    if (!inc_arg) {
        inc_arg = hash_include_new(arg,arglen);
        if (sym_def) {
            hash_include_define(inc_arg,sym_def);
        }
        hash_include_add(inc_arg);
        report_include(inc_arg);
    } else {
        canonical_string_const_h old_def = hash_include_definition(inc_arg);
        if (sym_def) {
            if (!old_def || !canonical_string_equal(sym_def,old_def)) {
                hash_include_define(inc_arg,sym_def);
            }
        } else if (old_def) {
            hash_include_undefine(inc_arg);
        }
        if (!GET_PUBLIC(args,list_only_once)) {
            report_include(inc_arg);
        }
    }
}

/* EOF*/
