/***************************************************************************
 *   Copyright (C) 2007-2010 Mike Kinghan, imk@strudl.org            *
 *   All rights reserved.                                                  *
 *                                                                         *
 *   Redistribution and use in source and binary forms, with or without    *
 *   modification, are permitted provided that the following conditions    *
 *   are met:                                                              *
 *                                                                         *
 *   Redistributions of source code must retain the above copyright        *
 *   notice, this list of conditions and the following disclaimer.         *
 *                                                                         *
 *   Redistributions in binary form must reproduce the above copyright     *
 *   notice, this list of conditions and the following disclaimer in the   *
 *   documentation and/or other materials provided with the distribution.  *
 *                                                                         *
 *   Neither the name of Symbian Software Ltd. nor the names of its        *
 *   contributors may be used to endorse or promote products derived from  *
 *   this software without specific prior written permission.              *
 *                                                                         *
 *   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS   *
 *   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT     *
 *   LIMITED TO, THE IMPLIED WARRANTIES OF  MERCHANTABILITY AND FITNESS    *
 *   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE        *
 *   COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,   *
 *   INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,  *
 *   BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS *
 *   OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED    *
 *   AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,*
 *   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF *
 *   THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH  *
 *   DAMAGE.                                                               *
 *                                                                         *
 **************************************************************************/

#include "line_select.h"
#include "args.h"
#include "if_control.h"
#include "directives_tally.h"

/*! \file line_select.c
 * \ingroup line_select_component line_select_interface line_select_internals
 *
 * This file implements the Line Select component
*/

/*! \ingroup line_select_internals_state_utils*/
/*@{*/
NO_PRIVATE_STATE(line_select);
IMPLEMENT(line_select,ZERO_INITABLE);
/*@}*/


/* API */

bool
dropping_line(void)
{
    return dead_line() ^ GET_PUBLIC(args,complement);
}

bool
eligible_line(int cmd_code, int directive)
{
    bool verdict = false;
    if (((1 << (cmd_code - 1)) & GET_PUBLIC(args,command)->cmd_mask) != 0) {
        if (GET_PUBLIC(args,list_only_inactive)) {
            verdict = GET_PUBLIC(line_select,dropping_line);
        } else if (GET_PUBLIC(args,list_only_active)) {
            verdict = !GET_PUBLIC(line_select,dropping_line);
        } else {
            verdict = true;
        }
        if (verdict && cmd_code == CMD_SYMBOLS) {
            switch(directive) {
            case HASH_IFDEF:
            case HASH_IFNDEF:
            case HASH_IF:
                verdict = GET_PUBLIC(args,list_symbols_in_ifs);
                break;
            case HASH_DEFINE:
                verdict = GET_PUBLIC(args,list_symbols_in_defs);
                break;
            case HASH_UNDEF:
                verdict = GET_PUBLIC(args,list_symbols_in_undefs);
                break;
            case HASH_INCLUDE:
                verdict = GET_PUBLIC(args,list_symbols_in_includes);
                break;
            default:
                ;
            }
        }
    }
    return verdict;
}


/* EOF */
