/***************************************************************************
 *   Copyright (C) 2007-2010 Mike Kinghan, imk@strudl.org            *
 *   All rights reserved.                                                  *
 *                                                                         *
 *   Contributed originally by Mike Kinghan, imk@strudl.org                *
 *                                                                         *
 *   Redistribution and use in source and binary forms, with or without    *
 *   modification, are permitted provided that the following conditions    *
 *   are met:                                                              *
 *                                                                         *
 *   Redistributions of source code must retain the above copyright        *
 *   notice, this list of conditions and the following disclaimer.         *
 *                                                                         *
 *   Redistributions in binary form must reproduce the above copyright     *
 *   notice, this list of conditions and the following disclaimer in the   *
 *   documentation and/or other materials provided with the distribution.  *
 *                                                                         *
 *   Neither the name of Symbian Software Ltd. nor the names of its        *
 *   contributors may be used to endorse or promote products derived from  *
 *   this software without specific prior written permission.              *
 *                                                                         *
 *   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS   *
 *   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT     *
 *   LIMITED TO, THE IMPLIED WARRANTIES OF  MERCHANTABILITY AND FITNESS    *
 *   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE        *
 *   COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,   *
 *   INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,  *
 *   BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS *
 *   OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED    *
 *   AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,*
 *   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF *
 *   THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH  *
 *   DAMAGE.                                                               *
 *                                                                         *
 **************************************************************************/
#include "unconfigured_symbols.h"
#include "args.h"
#include "report.h"


/*!\ingroup unconfigured_symbols_component, unconfigured_symbols_interface,
unconfigured_symbols_internals
	\file unconfigured_symbols.c
	This file implements the Configured Symbols component
*/

/*!\ingroup unconfigured_symbols_internals_state_utils*/
/*@{*/
NO_PRIVATE_STATE(unconfigured_symbols);
/*@}*/

/*!\addtogroup unconfigured_symbols_internals_state_utils*/
/*@{*/
IMPLEMENT(unconfigured_symbols,USER_INITABLE);

DEFINE_USER_INIT(unconfigured_symbols)(STATE_T(unconfigured_symbols) * st_state)
{
    st_state->sym_tab = ptr_set_new((comparator_t)symbol_compare,
                                    (dtor_t)symbol_dispose,
                                    (cloner_t)symbol_copy);
}

DEFINE_USER_FINIS(unconfigured_symbols)(STATE_T(unconfigured_symbols) *
                                        st_state)
{
    ptr_set_dispose(st_state->sym_tab);
}
/*@}*/

/* Helpers */

/*! \addtogroup unconfigured_symbols_internals*/
/*@{*/

/*!	Lookup the unconfigured symbols table for a symbol name.
	\param	name	The symbol name.
	\param	namelen	The length of the name.
	\return If a symbol with the given name is found then it is returned.
	Otherwise a new symbol with that name. A new symbol can be distinguished
	by calling \e symbol_seen(), which will return true if the symbol is
	new.
*/
static symbol_h
unconfigured_symbol_lookup(char const *name,size_t namelen)
{
    symbol_h sym = ptr_set_search(	GET_PUBLIC(unconfigured_symbols,sym_tab),
                                    name,namelen);
    if (!sym) {
        sym = symbol_new(name,namelen);
    }
    return sym;
}

/*!	Add a symbol to the unconfigured symbols table.
	\param	sym	The symbol to be added.
*/
static void
unconfigured_symbol_add(symbol_h sym)
{
    symbol_memorise(sym);
    ptr_set_insert(	GET_PUBLIC(unconfigured_symbols,sym_tab),sym);
}

/*@}*/

/* API *********************************************************************/

void
unconfigured_symbol_tally(canonical_string_const_h name)
{
    symbol_h sym;
    char const * text;
    size_t len;
    assert(name);
    text = canonical_string_text(name);
    len = strlen(text);
    assert(text && len);
    sym = unconfigured_symbol_lookup(text,len);
    if (!symbol_seen(sym)) {
        report_symbol(sym);
        unconfigured_symbol_add(sym);
    } else if (!GET_PUBLIC(args,list_only_once)) {
        report_symbol(sym);
    }
}


/* EOF*/
